@@ -1,12 +1,17 @@
 package Gtk2::Ex::HyperTextBuffer;
 
-our $VERSION = '0.15';
+our $VERSION = '0.17';
 
 use strict;
 use Gtk2;
 use Gtk2::Pango; # pango constants
 use File::Spec;  # for locating images
-use Glib::Object::Subclass Gtk2::TextBuffer:: ;
+use Glib::Object::Subclass
+	Gtk2::TextBuffer::,
+	signals => {
+		# new signals
+		edit_mode_changed => { },
+	} ;
 
 =head1 NAME
 
@@ -27,9 +32,7 @@ instead of plain text. It also keeps an undo-/redo-stack.
 
 =cut
 
-# It seems to me that 10 * PANGO_SCALE is the normal font size
 # TODO make tag styles configable
-# FIXME replace "weight" with "scale" ?
 our %TAGS = (
 	bold      => [weight => PANGO_WEIGHT_BOLD],
 	italic    => [style => 'italic'],
@@ -38,11 +41,11 @@ our %TAGS = (
 	verbatim  => [family => 'monospace'], # wrap_mode => 'none'], # Bug withwrap_mode !?
 	Verbatim  => [family => 'monospace', wrap_mode => 'none', indent => 40],
 	link      => [foreground => 'blue', underline => 'single'],
-	head1     => [weight => PANGO_WEIGHT_BOLD, size => 10*1.2**3 * PANGO_SCALE ],
-	head2     => [weight => PANGO_WEIGHT_BOLD, size => 10*1.2**2 * PANGO_SCALE ],
-	head3     => [weight => PANGO_WEIGHT_BOLD, size => 12 * PANGO_SCALE        ],
-	head4     => [weight => PANGO_WEIGHT_BOLD, size => 10 * PANGO_SCALE        ],
-	head5     => [style  => 'italic', weight => PANGO_WEIGHT_BOLD, size => 10 * PANGO_SCALE],
+	head1     => [weight => PANGO_WEIGHT_BOLD, scale => 1.15**4 ],
+	head2     => [weight => PANGO_WEIGHT_BOLD, scale => 1.15**3 ],
+	head3     => [weight => PANGO_WEIGHT_BOLD, scale => 1.15**2 ],
+	head4     => [weight => PANGO_WEIGHT_ULTRABOLD, scale => 1.15 ],
+	head5     => [weight => PANGO_WEIGHT_BOLD, scale => 1.15, style  => 'italic'],
 );
 
 our @_tags = grep {$_ ne 'link'} keys %TAGS;
@@ -54,6 +57,7 @@ our @_targets = (
 	{ target => 'TEXT'          },
 	{ target => 'COMPOUND_TEXT' },
 	{ target => 'UTF8_STRING'   },
+	{ target => 'text/plain'    },
 	{ target => 'ZIM_TREE'      },
 );
 
@@ -70,14 +74,12 @@ Simple constructor.
 sub INIT_INSTANCE {
 	my $self = shift;
 	$self->{edit_mode_tags} = [];
-	$self->{image_marks} = [];
-#	$self->signal_connect(mark_set => sub {
-#			my ($self, $iter, $mark) = @_;
-#			return unless $mark->get_name eq 'insert';
-#			my @names = map {$_->get_property('name')} $self->_get_open_tags($iter);
-#			warn "Tags: @names\n";
-#			#$self->set_edit_mode_from_cursor;
-#		} );
+	$self->signal_connect(mark_set => sub {
+			my ($self, $iter, $mark) = @_;
+			$self->set_edit_mode_from_cursor()
+				if $mark->get_name eq 'insert';
+		} );
+	$self->signal_connect_after(insert_text => \&on_insert);
 }
 
 =item C<clear()>
@@ -85,7 +87,7 @@ sub INIT_INSTANCE {
 Deletes all buffer contents.
 
 Be aware that this module sometimes stores a reference to the buffer in a
-global variable in order to get copy-pasting with mark up work. This means
+global variable in order to get copy-pasting with markup work. This means
 you could save a lot of memory when you always clear the object when you are
 done with it.
 
@@ -94,7 +96,6 @@ done with it.
 sub clear {
 	my $self = shift;
 	$self->{edit_mode_tags} = [];
-	$self->{image_marks} = [];
 	$self->delete( $self->get_bounds );
 }
 
@@ -145,9 +146,14 @@ Insert one or more partial parse trees in the buffer at ITER.
 sub insert_blocks {
 	my $self = shift;
 	my $iter = shift;
+	my $insert = $self->get_insert;
+	my $mark = $self->create_mark(
+		'insert-blocks-orig-insert',
+		$self->get_iter_at_mark($insert),
+		1 );
 	$self->place_cursor($iter);
-	$self->set_edit_mode_from_cursor;
 	$self->insert_blocks_at_cursor(@_);
+	$self->place_cursor( $self->get_iter_at_mark($mark) );
 }
 
 =item C<insert_blocks_at_cursor(BLOCK, ...)>
@@ -161,10 +167,17 @@ my $_Verbatim = 0;
 sub insert_blocks_at_cursor {
 	# Make sure not to modify content of the reference !
 	# TODO: check default editable / region editable
+	my $self = shift;
+	
+	$self->signal_handlers_block_by_func(\&on_insert);
+	$self->_insert_blocks(@_);
+	$self->signal_handlers_unblock_by_func(\&on_insert);
+	return 1;
+}
+
+sub _insert_blocks {
 	my ($self, @blocks) = @_;
-#	$self->set_edit_mode_from_cursor
-#		unless defined $self->{edit_mode_tags}; # FIXME
-	$self->{edit_mode_tags} ||= [];
+
 	for (@blocks) {
 		unless (ref $_) {
 			$_ =~ s/^(\s*)\*(\s+)/$1\x{2022}$2/mg unless $_Verbatim; # FIXME doesn't belong in this module !
@@ -191,7 +204,7 @@ sub insert_blocks_at_cursor {
 			if ($tag) {
 				$_Verbatim++ if $type eq 'Verbatim'; # FIXME
 				push @{$self->{edit_mode_tags}}, $tag;
-				$self->insert_blocks_at_cursor(@$_[2 .. $#$_]); # recurs
+				$self->_insert_blocks(@$_[2 .. $#$_]); # recurs
 				pop @{$self->{edit_mode_tags}};
 				$_Verbatim-- if $type eq 'Verbatim'; # FIXME
 			}
@@ -201,7 +214,7 @@ sub insert_blocks_at_cursor {
 					$self->{base} =~ s#^file://##;
 					#warn "base set to: $self->{base}\n";
 				}
-				$self->insert_blocks_at_cursor(@$_[2 .. $#$_]); # recurs
+				$self->_insert_blocks(@$_[2 .. $#$_]); # recurs
 			}
 			else { die "Unknown tag in parse tree: $type" }
 
@@ -211,36 +224,29 @@ sub insert_blocks_at_cursor {
 		}
 	}
 
-	return 1;
 }
 
-=item C<insert_image_from_file(FILENAME)>
+=item C<insert_image_from_file(FILENAME, ITER)>
 
-Insert an image in the buffer at the cursor.
+Insert an image in the buffer at the cursor
+or at ITER if ITER is specified.
 
 =cut
 
 sub insert_image_from_file {
-	my ($self, $src) = @_;
+	my ($self, $src, $iter) = @_;
 	my $file = $self->{base} ? File::Spec->rel2abs($src, $self->{base}) : $src;
-	#print "file: $$meta{src}, base: $self->{base} => $file\n";
-
-	my $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($file) if -f $file and -r _;
+	#warn "file: $src, base: $self->{base} => $file\n";
 
+	my $pixbuf = undef;
+	eval { $pixbuf = Gtk2::Gdk::Pixbuf->new_from_file($file) }
+		if (-f $file and -r _) ;
 	$pixbuf ||= Gtk2::Image->new->render_icon('gtk-missing-image', 'dialog')->copy;
-	die "could not insert image: $file\n" unless $pixbuf;
-	
-	if (defined $pixbuf->{image_src}) {
-		# copy does not always copy, or so it seems :(
-		$pixbuf = $pixbuf->copy;
-		warn "WARNING: pixbuf bug !??";
-	}
-	die "ERROR: Pixbuf BUG !" if defined $pixbuf->{image_src};
-	#warn "PixBuf: $pixbuf ($pixbuf->{image_src})\n";
+	die "Could not insert image: $file\n" unless $pixbuf;
 	
 	$pixbuf->{image_src} = $src;
 	$pixbuf->{image_file} = $file;
-	my $iter = $self->get_iter_at_mark( $self->get_insert );
+	$iter ||= $self->get_iter_at_mark( $self->get_insert );
 
 	#my $mark = $self->create_mark(undef, $iter, 0);
 	#$mark->set_visible(1);
@@ -337,7 +343,6 @@ sub get_parse_tree {
 			$$next[2] =~ s/^(\n*)//;
 			$lines += length $1;
 			splice @$next, 2, 1 unless length $$next[2];
-			$$root[$_+1] = undef unless @{$$root[$_+1]} > 2;
 		}
 		elsif (defined $$root[$_+1]) {
 			$$root[$_+1] =~ s/^(\n*)//;
@@ -360,10 +365,17 @@ sub get_parse_tree {
 		
 		$$root[$_][1]{empty_lines} = $lines - 1 if $lines > 1;
 	}
-	@$root = grep defined($_), @$root;
+
+	# Filter out empty tags
+	@$root = grep {
+		defined($_) and
+		(ref($_) and ($$_[0] !~ /^(link|image)/)) 
+			? (@$_ > 2)
+			: length($_)
+	} @$root;
 		
 	unshift @$root, 'Document', {};
-	#use Data::Dumper; print STDERR Dumper $root;
+	#use Data::Dumper; warn "get_parse_tree: ", Dumper $root;
 	return $root;
 }
 
@@ -452,6 +464,7 @@ sub select_line {
 	$begin = $self->get_iter_at_line( $begin->get_line )
 		unless $begin->starts_line;
 	while (! $iter->ends_line) { $iter->forward_char }
+	return unless $iter->compare($begin) == 1;
 	$self->select_range($begin, $iter);
 	return ($begin, $iter);
 }
@@ -531,9 +544,10 @@ sub replace_selection {
 	return $start, $end;
 }
 
-=item C<set_edit_mode_tags>
+=item C<set_edit_mode_tags(TAG, ..)>
 
-TODO
+Set the current edit mode to a certain list of tags. This means that all
+text you insert will get these tags applied.
 
 =cut
 
@@ -541,15 +555,31 @@ sub set_edit_mode_tags {
 	my $self = shift;
 	my $table = $self->get_tag_table;
 	$self->{edit_mode_tags} =
-		[ map { ref($_) ? $_ : $table->lookup($_) } @$_ ];
+		[ map { ref($_) ? $_ : $table->lookup($_) } @_ ];
+	$self->signal_emit('edit_mode_changed');
 }
 
-=item C<set_edit_mode_from_cursor>
+=item C<set_edit_mode_from_cursor()>
 
-TODO
+Like C<set_edit_mode_tags()> but copies tags from the cursor position.
+This function is called automaticly after moving the cursor.
 
 =cut
 
+sub set_edit_mode_from_cursor {
+	my $self = shift;
+	my $iter = $self->get_iter_at_mark( $self->get_insert );
+	$self->{edit_mode_tags} = [$self->_get_open_tags($iter)];
+	my $string = "@{$self->{edit_mode_tags}}";
+	my $changed = ($string ne $self->{_edit_mode_string});
+	$self->{_edit_mode_string} = $string;
+	$self->signal_emit('edit_mode_changed') if $changed;
+	#warn join ' ', "Mode:", map(
+	#	{$_->{is_link} ? 'link' : $_->get_property('name')}
+	#	@{$self->{edit_mode_tags}}
+	#), "\n" if $changed;
+}
+
 sub _get_open_tags {
 	# iter->get_tags has "left" gravity
 	# this function has "right" gravity
@@ -559,21 +589,13 @@ sub _get_open_tags {
 		@tags = grep {$_ ne $t} @tags;
 	}
 	push @tags, $iter->get_toggled_tags(0);
+	@tags = grep {$_->get_property('name') !~ /gtkspell/} @tags;
 	return @tags;
 }
 
-sub set_edit_mode_from_cursor {
-	my $self = shift;
-	my $iter = $self->get_iter_at_mark( $self->get_insert );
-	$self->{edit_mode_tags} = [$iter->get_tags];
-	#print join ' ', "Mode:", map(
-	#	{$_->get_property('name')} @{$self->{edit_mode_tags}}
-	#), "\n";
-}
+=item C<get_edit_mode_tags()>
 
-=item C<get_edit_mode_tags>
-
-TODO
+Returns a list of tags that will be applied to newly entered text.
 
 =cut
 
@@ -583,32 +605,30 @@ sub get_edit_mode_tags {
 	@{$_[0]->{edit_mode_tags}};
 }
 
-=item C<toggle_edit_mode_tag>
+=item C<toggle_edit_mode_tag(TAG, BOOL)>
 
-TODO
+Toggles a tag for editing. If BOOL is undefined it just toggles, else it sets the tag to the value of BOOL.
 
 =cut
 
 sub toggle_edit_mode_tag {
-	my ($self, $tag) = @_;
+	my ($self, $tag, $bool) = @_;
 	$tag = $self->get_tag_table->lookup($tag) unless ref $tag;
-	$self->set_edit_mode_from_cursor
-		unless defined $self->{edit_mode_tags};
-	if (grep {$_ eq $tag} @{$self->{edit_mode_tags}}) {
-		@{$self->{edit_mode_tags}} = grep {$_ ne $tag} @{$self->{edit_mode_tags}};
-	}
-	else {
-		push @{$self->{edit_mode_tags}}, $tag;
-	}
+	my @tags = grep {$_ ne $tag} @{$self->{edit_mode_tags}};
+	push @tags, $tag
+		if @tags == @{$self->{edit_mode_tags}} and $bool != 0;
+	@{$self->{edit_mode_tags}} = @tags;
+	$self->signal_emit('edit_mode_changed');
 }
 
-=item C<reset_edit_mode>
-
-TODO
-
-=cut
-
-sub reset_edit_mode { $_[0]->{edit_mode_tags} = undef }
+sub on_insert {
+	my ($self, $end, $string) = @_;
+	# length in argument list does not understand utf8
+	my $start = $end->copy;
+	$start->backward_chars(length $string);
+	$self->remove_all_tags($start, $end);
+	$self->apply_tag($_, $start, $end) for @{$self->{edit_mode_tags}};
+}
 
 =item C<undo>
 
@@ -704,6 +724,9 @@ sub _dump_tree { # TODO add "\n" around headings and support lists
 	my $string = '';
 	$string .= ref($_) ? _dump_tree($_) : $_
 		for @$tree[2 .. $#$tree];
+	$string .= "\n" if $$tree[0] =~ /^head/;
+	$string .= "\n"x$$tree[1]{empty_lines}
+		if $$tree[1]{empty_lines};
 	return $string;
 }
 
@@ -788,20 +811,43 @@ sub _clipboard_paste_data {
 	else { $iter = $self->get_iter_at_mark( $self->get_insert ) }
 	
 	# actual insert
-	if ($paste_data->{type} eq 'ZIM_TREE') {
-		#use Data::Dumper; print STDERR '_clipboard: ', Dumper $_clipboard;
-		$self->insert_blocks($iter, $_clipboard);
-	}
+	my $tree;
+	if ($paste_data->{type} eq 'ZIM_TREE') { $tree = $_clipboard }
 	else { # text
-		$paste_data->{interactive}
-			? $self->insert_interactive(
-				$iter, $string, $paste_data->{editable} )
-			: $self->insert($iter, $string) ;
+		# FIXME url parsing copy-pasted from Zim/Formats/Wiki.pm
+		my $i = 0;
+		my @tree = ('Document', {},
+			map   { ($i++ % 2) ? _parse_link($_) : $_ }
+			split m#(
+				\b\w[\w\+\-\.]+://  \S*\[\S+\](?:\S+[\w\/])? |
+				\b\w[\w\+\-\.]+://  \S+[\w\/]                |
+				\bmailto:\S+\@      \S*\[\S+\](?:\S+[\w/])?  |
+				\bmailto:\S+\@      \S+[\w/]                 |
+				\b\S+\@\S+\.\w+\b
+			)#x, $string);
+			# The host name in an uri can be "[hex:hex:..]" for ipv6
+			# but we do not want to match "[http://foo.org]"
+			# See rfc3986 for the official -but unpractical- regex
+		$tree = \@tree;
 	}
 
+	$self->insert_blocks($iter, $tree);
 	$self->end_user_action if $paste_data->{interactive};
 }
 
+sub _parse_link { # FIXME copy-pasted from Zim/Formats/Wiki.pm
+	my $l = shift;
+	my $text;
+	if ($l =~ /^\[\[([^|]+)\|?(.*)\]\]$/) { # [[link]] or [[link|text]]
+		($l, $text) = ($1, $2);
+	}
+	if ($l =~ /^mailto:|^\S+\@\S+\.\w+$/) {
+		$text = $l unless length $text;
+		$l =~ s/^(mailto:)?/mailto:/;
+	}
+	return ['link', {to => $l}, length($text) ? $text : $l ];
+}
+
 1;
 
 __END__
