@@ -1,6 +1,6 @@
 package Gtk2::Ex::HyperTextView;
 
-our $VERSION = '0.15';
+our $VERSION = '0.17';
 
 use strict;
 use Gtk2;
@@ -48,6 +48,8 @@ but also with other TextBuffer classes.
 
 =item C<new_with_buffer(BUFFER)>
 
+Constructors.
+
 =cut
 
 # sub new() inherited from Glib::Object
@@ -67,6 +69,7 @@ sub INIT_INSTANCE {
 	$self->{hand_cursor}     = Gtk2::Gdk::Cursor->new('hand2');
 	$self->{link_properties} = [foreground => 'blue', underline => 'single'];
 	$self->{hovering_over_link} = 0;
+	$self->{follow_on_enter} = 1;
 	
 	$self->signal_connect_after('realize' => sub {
 		my ($view) = @_;
@@ -266,6 +269,133 @@ sub set_cursor_if_appropriate {
 	return $hovering ? $self->get_link_at_iter($iter, $link_tag) : undef;
 }
 
+# ###### #
+# Search #
+# ###### #
+
+=item C<search(STRING, DIRECTION)>
+
+Searches for STRING in the buffer and selects the next occurence it finds.
+The search is case-insensitive and TEXT can not include line breaks.
+The search wraps around when nothing is found before the end/begin of the buffer.
+
+DIRECTION can be 1, -1 for forward and backward search. If DIRECTION is 0 it will
+match forward including the current cursor. Try:
+
+	$entry->signal_connect( changed =>
+		sub { $htext->search( $entry->get_text, 0 ) }    );
+
+to get an entry that searches while typing.
+		
+Returns boolean for success.
+
+=cut
+
+sub search {
+	my ($self, $string, $direction) = @_;
+	my $regex = '(?i)' . join '\\s+', map quotemeta($_), split /\s+/, $string;
+	my $buffer = $self->get_buffer;
+	my $iter = $buffer->get_iter_at_mark( $buffer->get_insert );
+	if ($direction == 1)     { $iter->forward_char  }
+	elsif ($direction == -1) { $iter->backward_char }
+	my $offset = $iter->get_line_offset;
+	my $line = $iter->get_line;
+	my $last = $buffer->get_line_count - 1;
+
+	my $range;
+	if (! length $string) {} # do nothing
+	elsif ($direction != -1) { # forward
+		$range =  _search_lines($buffer, qr/(.{$offset,}?)($regex)/, $line)
+		       || _search_lines($buffer, qr/(.*?)($regex)/, $line+1 .. $last, 0 .. $line-1)
+		       || _search_lines($buffer, qr/(.{0,$offset}?)($regex)/, $line) ;
+	}
+	else { # backward
+		$range =  _search_lines($buffer, qr/(.{0,$offset})($regex)/, $line)
+		       || _search_lines($buffer, qr/(.*)($regex)/, reverse $line+1 .. $last, 0 .. $line-1)
+		       || _search_lines($buffer, qr/(.{$offset,})($regex)/, $line) ;
+	}
+	
+	my $succes = defined $range;
+	if ($succes) { $buffer->select_range(@$range) }
+	else { # unset selection
+		$buffer->move_mark(
+			$buffer->get_selection_bound,
+			$buffer->get_iter_at_mark( $buffer->get_insert ) );
+	}
+	
+	$self->scroll_mark_onscreen( $buffer->get_insert );
+	return $succes;
+}
+
+sub _search_lines {
+	my ($buffer, $regex, @lines) = @_;
+	for (@lines) {
+		my $begin = $buffer->get_iter_at_line($_);
+		next if $begin->ends_line;
+		my $end = $begin->copy;
+		$end->forward_to_line_end;
+		my $line = $begin->get_slice($end);
+		$line =~ $regex;
+		#warn "Matching $regex against \"$line\"\n";
+		if (defined $2) {
+			$begin->forward_chars(length $1);
+			$end = $begin->copy;
+			$end->forward_chars(length $2);
+			return [$begin, $end];
+		}
+	}
+	return undef;
+}
+
+=item C<replace(STRING)>
+
+Replace the current selection with STRING.
+To be used in combination with C<search()> to create a
+"Find and Replace" function.
+
+=cut
+
+sub replace {
+	my ($self, $string) = @_;
+	my $buffer = $self->get_buffer;
+	my ($start, $end) = $buffer->get_selection_bounds;
+	return if !$end or $start == $end;
+
+	my $offset = $start->get_offset;
+	$buffer->delete($start, $end);
+	$start = $buffer->get_iter_at_offset($offset);
+	$buffer->insert($start, $string);
+}
+
+=item C<replace_all(OLD, NEW)>
+
+Loop through the document once and replace all matches for OLD with NEW.
+
+=cut
+
+sub replace_all {
+	my ($self, $old, $new) = @_;
+	my $buffer = $self->get_buffer;
+
+	my $insert = $buffer->get_insert;
+	my $iter = $buffer->get_iter_at_mark( $insert );
+	my $mark = $buffer->create_mark('replace_all_mark', $iter, 1);
+	my $i = 0;
+	my $wrap = 0;
+	while ( $self->search($old, $i++) ) {
+		my ($match) = $buffer->get_selection_bounds;
+		$wrap ||= ($match->compare($iter) == -1);
+		if ($wrap) {
+			my $end = $buffer->get_iter_at_mark($mark);
+			last if $match->compare($end) >= 0;
+		}
+		$self->replace($new);
+		$iter = $buffer->get_iter_at_mark( $insert );
+	}
+	$buffer->delete_mark($mark);
+	return $i;
+}
+
 # ###### #
 # Events #
 # ###### #
@@ -319,7 +449,15 @@ sub on_key_press_event {
 	if ($val == $k_return or $val == $k_kp_enter) { # Enter
 		my $buffer = $self->get_buffer;
 		my $iter = $buffer->get_iter_at_mark($buffer->get_insert());
-		return 1 if defined $self->click_if_link_at_iter($iter);
+		if ($self->{follow_on_enter} or $event->state >= 'mod1-mask' ) {
+			# follow link
+			return 1 if defined $self->click_if_link_at_iter($iter);
+		}
+		else {
+			# ignore <enter> within link
+			my ($l) = grep {$_->{is_link}} $iter->get_tags;
+			return 1 if defined($l) and ! $iter->toggles_tag($l);
+		}
 	}
 #	else { printf "key %x pressed\n", $val } # perldoc -m Gtk2::Gdk::Keysyms
 	
