@@ -1,8 +1,10 @@
 package Zim::Formats::Html;
 
 use strict;
+use Encode;
+use Zim::Template;
 
-our $VERSION = '0.15';
+our $VERSION = '0.17';
 
 =head1 NAME
 
@@ -22,43 +24,85 @@ Converts a parse tree into plain text.
 
 =cut
 
+our %entities = (
+	'&' => 'apos',
+	'<' => 'lt',
+	'>' => 'gt',
+);
+	
 sub save_tree {
 	my ($class, $io, $tree, $page) = @_;
 
 	my $template = $page->{repository}{_template};
-	die "Html format needs a template\n" unless $template;
-	my ($head, $foot) = split /\[\%\s*BODY\s*\%\]/, _read_file($template), 2;
-	my $title = $page->basename;
+	$template = length($template)
+		? Zim::Template->new($template)
+		: Zim::Template->new(\'[% page.body %]')
+		unless ref $template ;
+
+	my ($title, $heading) = ($page->basename, '');
 	if (ref $$tree[2] and $$tree[2][0] =~ /^head/) {
-		# FIXME better logic for this ?
+		# FIXME This logic belongs somewhere central
 		$title = $$tree[2];
 		$$tree[2] = '';
 		$title = join '', @$title[2 .. $#$title];
+		$heading = $title;
 	}
-	my $prev = $page->{repository}{_prev}->name if $page->{repository}{_prev};
-	my $next =  $page->{repository}{_next}->name if $page->{repository}{_next};
+	# else above defaults
+
 	my @path = $page->namespaces;
 	my $root = '../' x scalar @path || './';
 	$page->{_root_dir} = $root;
-	for my $p ($head, $foot) {
-		$p =~ s/\[\%\s*$$_[0]\s*\%\]/$$_[1]/ig for
-			[TITLE       => $title              ],
-			[PREV        => $prev               ],
-			[PREV_FILE   => _href($page, $prev) ],
-			[NEXT        => $next               ],
-			[NEXT_FILE   => _href($page, $next) ],
-			[ZIM_VERSION => $Zim::VERSION       ],
-			[ROOT_DIR    => $root               ] ;
-	}
-	
-	my $old_fh = select $io;
-	eval {
-		print $head;
-		_dump_node($tree, $page);
-		print $foot;
+
+	my $data = {
+		page => {
+			name => '',
+			namespace => '',
+			title => $title,
+			heading => $heading,
+			links => sub {
+				[ map { {name => $_, file => _href($page, $_)} }
+					sort $page->_list_links ]
+			},
+			backlinks => sub {
+				return [ map { {name => $_, file => _href($page, $_)} }
+					sort $page->_list_backlinks ] ;
+			},
+		},
+		prev => {},
+		next => {},
+		repository => { root => $root },
+		zim => { version => $VERSION },
 	};
-	select $old_fh;
-	die $@ if $@;
+	$data->{page}{body} = sub {
+		my (undef, $fh) = @_;
+		return unless $fh;
+		my $old_fh = select $fh;
+		eval {
+			$page->{_mode} = lc $data->{mode};
+			print qq#<div class="slide">\n# if $page->{_mode} eq 's5';
+			_dump_node($tree, $page);
+			print qq#</div>\n# if $page->{_mode} eq 's5';
+		};
+		select $old_fh;
+		die $@ if $@;
+		return undef; # no caching here
+	};
+	if ($page->{repository}{_prev}) {
+		my $prev = $page->{repository}{_prev}->name;
+		$data->{prev} = {
+			name => $prev,
+			file => _href($page, $prev),
+		};
+	}
+	if ($page->{repository}{_next}) {
+		my $next = $page->{repository}{_next};
+		$data->{next} = {
+			name => $next,
+			file => _href($page, $next),
+		};
+	}
+
+	$template->process($data => $io);
 }
 
 sub _dump_node {
@@ -70,30 +114,37 @@ sub _dump_node {
 	my @tags;
 	if ($name =~ /^head(\d+)/) { @tags = ("<h$1>", "</h$1>\n") }
 	elsif ($name eq 'Para') {
-		@tags = ("<p>\n", "</p>\n");
+		@tags = ("<p>\n", "\n</p>\n");
 		if (! ref $$node[0] and $$node[0] =~ /^\s*\*\s+/m) {
 			# ugly hack ... should not be necessary
 			parse_list($page, @$node);
 			@$node = ();
 		}
+		elsif ( $page->{_mode} eq 's5'
+				and ! ref $$node[0]
+				and $$node[0] =~ s/^----+\s+//
+		) {
+			print qq#</div>\n\n\n<div class="slide">\n#;
+			shift @$node unless $$node[0] =~ /\S/;
+		}
 	}
-	elsif ($name eq 'Verbatim') {
-		@tags = ("<blockquote>\n<pre>\n", "</pre>\n</blockquote>\n")
-	}
+	elsif ($name eq 'Verbatim' ) {  @tags = ("<pre>\n", "</pre>\n") }
 	elsif ($name eq 'bold'     ) { @tags = ('<b>', '</b>')   }
 	elsif ($name eq 'italic'   ) { @tags = ('<i>', '</i>')   }
 	elsif ($name eq 'underline') { @tags = ('<u>', '</u>')   }
-	elsif ($name eq 'strike')    { @tags = ('<strike>', '</strike>') }
+	elsif ($name eq 'strike'   ) { @tags = ('<strike>', '</strike>') }
 	elsif ($name eq 'verbatim' ) { @tags = ('<tt>', '</tt>') }
 	elsif ($name eq 'link') {
 		my ($type, $href) = $page->parse_link($$opt{to});
-		$href = _href($page, $page->resolve_link($$opt{to})->name)
-			if $type eq 'page';
+		if ($type eq 'page') {
+			my $link = $page->resolve_link($$opt{to});
+			$href = _href($page, $link->name) if $link;
+		}
 		print STDERR "  link: $$opt{to} => $href\n";
 		@tags = ("<a href='$href'>", "</a>");
 	}
 	elsif ($name eq 'image') {
-		@tags = ("<div><img src='$$opt{src}' /></div>", '');
+		@tags = ("<img src='$$opt{src}' alt='$$opt{src}' />", '');
 	}
 	# else recurs silently
 
@@ -101,7 +152,8 @@ sub _dump_node {
 	for (@$node) {
 		if (ref $_) { _dump_node($_, $page) } # recurse
 		else {
-			$_ =~ s/([^-\n\t !\#\$\%\(\)\*\+,\.\~\/\:\;=\?\@\[\\\]\^_\`\{\|\}abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789])/'&#'.(ord($1)).';'/eg; # RegEx copy pasted from Pod::Simple::Html
+			s#([<>&])#\&$entities{$1}\;#g; # encode
+			s#\n#<br />\n#g unless $tags[0] =~ m#<pre>#;
 			print $_;
 		}
 	}
@@ -110,6 +162,7 @@ sub _dump_node {
 
 sub _href {
 	my ($page, $href) = @_;
+	return '' unless length $href;
 	my $namespace = $page->namespace;
 	#print STDERR "page: $href, relative: $namespace => ";
 	$href =~ s#^:*#:#;
@@ -121,6 +174,8 @@ sub _href {
 	$href =~ s#[/:]+#/#g;
 	$href .= '.html';
 	#print STDERR $href."\n";
+	$href = Encode::encode_utf8($href); # turn utf8 flag off
+	$href =~ s#([^a-zA-Z0-9/\.])#sprintf '%%%02x', ord $1#eg; # url encoding
 	return $href;
 }
 
@@ -134,7 +189,7 @@ sub parse_list { # FIXME totally wrong level for this kind of parsing :(
 	for (@nodes) {
 		if (ref $_) { _dump_node($_, $page) }
 		else {
-			$_ =~ s/([^-\n\t !\#\$\%\(\)\*\+,\.\~\/\:\;=\?\@\[\\\]\^_\`\{\|\}abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789])/'&#'.(ord($1)).';'/eg; # RegEx copy pasted from Pod::Simple::Html
+			s#([<>&])#\&$entities{$1}\;#g; # encode
 			my @lines = split /[\n\r]+/, $_;
 			for (@lines) {
 				if ($_ =~ s/^(\s*)\*\s+//) {
@@ -158,14 +213,6 @@ sub parse_list { # FIXME totally wrong level for this kind of parsing :(
 	print "</ul>\n" for 1 .. $level + 1;
 }
 
-sub _read_file {
-	my $file = shift;
-	open FILE, "<$file" or die "Could not read file: $file\n";
-	my $text = join '', <FILE>;
-	close FILE;
-	return $text;
-}
-
 1;
 
 __END__
