@@ -2,7 +2,7 @@ package Zim::Page;
 
 use strict;
 
-our $VERSION = '0.14';
+our $VERSION = '0.17';
 
 use overload
 	'""' => sub { $_[0]->{name} }, # \&name results in bugs ?
@@ -182,24 +182,46 @@ sub delete { $_[0]->{repository}->delete_page(@_) }
 
 =item C<parse_link(LINK)>
 
-Returns the link type and link target for LINK.
-The type can be 'page', 'file', 'url' or 'mail'.
+Returns the link type and link target for LINK. The type can be 'page',
+'file', 'mail' or 'man' - for urls the protocol is used as type.
+
+The target returned is usually the same as LINK but can be different
+for example for interwiki links.
 
 =cut
 
 sub parse_link {
 	my ($self, $link) = @_;
-	if ($link =~ s/^([\w.]+)>//) { # interwiki link
-		$link = $self->{repository}->root->interwiki_lookup($1, $link);
-		return unless defined $link;
+	if ($link =~ s/^(\w[\w\+\-\.]+)\?//) { # interwiki link
+		my $key = $1;
+		my $l = $self->{repository}->root->interwiki_lookup($key, $link);
+		if (defined $l) { $link = $l }
+		else {
+			return 'man', $link if $key eq 'man'; # man?zim should work
+			return undef;
+		}
 	}
 	$_ = $link;
-	my $type = m#^file://#  ? 'file' :
-	           m#^(\w+)://# ? 'url'  :
-		   m#^mailto:#  ? 'mail' :
-		   m#/#         ? 'file' : 'page' ;
+	
+	my $type;
+	if    (m#^(\w+[\w\+\-\.]+)://#) { # urls
+		$type = $1;
+		$type = 'file' if $^O eq 'MSWin32' and $type eq 'file';
+	}
+	elsif (m#^mailto:|^\S+\@\S+\.\w+$#) { # email
+		$link =~ s#^(mailto:)?#mailto:#;
+		$type = 'mail';
+	}
+	else {
+		$type =
+			m#/#		? 'file' :
+			m#:#		? 'page' :
+			m#\(\d\w*\)$#	? 'man'  : 'page' ;
+	}
+
 	$link = Zim::File->abs_path($link, $self->{properties}{base})
 		if $type eq 'file';
+
 	return $type, $link;
 }
 
@@ -215,6 +237,33 @@ sub resolve_link { # 'cloning' hack needed when exporting
 	$obj->{repository}->resolve_link(@_)
 }
 
+=item C<match_word(WORD)>
+
+TODO: stable api for this
+
+=cut
+
+sub match_word { # TODO optimize by caching found links
+	my ($self, $word) = @_;
+	return $self->{repository}->can('_match_word')
+		? $self->{repository}->_match_word($self, $word)
+		: undef ;
+}
+
+=item C<equals(PAGE)>
+
+Check if PAGE is refering to the same page we are.
+This does not guarantee that the actual content is the same.
+
+=cut
+
+sub equals {
+	my ($self, $page) = @_;
+	return $page eq $self->{name} unless ref $page; 
+	return 0 if $page->{repository} ne $self->{repository};
+	return $self->{name} eq $page->name;
+}
+
 =item C<clone(SOURCE)>
 
 Import content from object SOURCE into this object.
@@ -225,6 +274,45 @@ by both SOURCE and TARGET.
 
 sub clone { die "Can not copy object of class ".ref($_[0])."\n" }
 
+=item C<get_next()>
+
+Returns next page.
+
+=item C<get_prev()>
+
+Return previous page.
+
+=cut
+
+sub get_next {
+	my $self = shift;
+	my ($ns, $name) = ($self->namespace, $self->basename);
+	my $prev;
+	for ($self->{repository}->list_pages($ns)) {
+		s/:$//;
+		return $self->{repository}->get_page($ns.$_)
+			if $prev eq $name;
+		$prev = $_;
+	}
+	return undef;
+}
+
+sub get_prev {
+	my $self = shift;
+	my ($ns, $name) = ($self->namespace, $self->basename);
+	my $prev;
+	for ($self->{repository}->list_pages($ns)) {
+		s/:$//;
+		if ($_ eq $name) {
+			return $self->{repository}->get_page($ns.$prev)
+				if length $prev;
+			last;
+		}
+		$prev = $_;
+	}
+	return undef;
+}
+
 1;
 
 __END__
