@@ -1,6 +1,6 @@
-my $app = Zim->current();
+my $app = Zim::GUI->current();
 
-eval 'use Zim::Components::TrayIcon';
+eval 'use Zim::GUI::TrayIcon';
 if ($@) {
 	$app->error_dialog(
 		"Failed to load TrayIcon plugin,\n".
@@ -9,7 +9,7 @@ if ($@) {
 	return 1;
 }
 
-my $trayicon = Zim::Components::TrayIcon->new(app => $app);
+my $trayicon = Zim::GUI::TrayIcon->new(app => $app);
 $app->add_object(TrayIcon => $trayicon);
 
 1;
